@@ -15,18 +15,13 @@ __email__ = "mbkumar@gmail.com"
 __status__ = "Alpha"
 __date__ = "6/4/14"
 
-import sys
 import math
-import json
 import copy
 import numpy as np
-import scipy
 
 from monty.dev import requires
+from monty.fractions import gcd
 
-from pymatgen.analysis.defects import Vacancy
-from pymatgen.core.structure import Composition
-from pymatgen.serializers.json_coders import PMGJSONEncoder, PMGJSONDecoder
 
 try:
     from sympy import Symbol, nsolve, Integer, Rational, Matrix, exp, solve, Eq
@@ -37,24 +32,6 @@ except ImportError:
 # physical consts
 k_B=8.6173324e-5                # eV/K
 
-def gcd(a, b):
-    """Return greatest common divisor using Euclid's Algorithm."""
-    while b:
-        a, b = b, a % b
-    return a
-
-def lcm(a, b):
-    """Return lowest common multiple."""
-    return a * b // gcd(a, b)
-
-def lcmm(*args):
-    """Return lcm of args."""
-    return reduce(lcm, args)
-
-def gcdd(*args):
-    """ Return gcd of args."""
-    return reduce(gcd, args)
-
 # Check the inputs
 def check_input(def_list):
     flag = True
@@ -125,7 +102,7 @@ def dilute_solution_model(structure, e0, vac_defs, antisite_defs, T,
 
     # Reduce the system and associated parameters such that only distinctive
     # atoms are retained
-    comm_div = gcdd(*tuple(multiplicity))
+    comm_div = gcd(*tuple(multiplicity))
     multiplicity = [val/comm_div for val in multiplicity]
     e0 = e0/comm_div
     T = Integer(T)
@@ -424,8 +401,8 @@ def dilute_solution_model(structure, e0, vac_defs, antisite_defs, T,
     single array"""
     plot_data['x'] = [dat[0][0] for dat in res1]         # x-axis data
     # Element whose composition is varied. For x-label
-    plot_data['x_label'] = "% "+els[0]                   
-    plot_data['y_label'] = "Point defect density"
+    plot_data['x_label'] = els[0]+ " mole fraction" 
+    plot_data['y_label'] = "Point defect concentration"
     conc = []
     for i in range(n):
         conc.append([])
