@@ -22,7 +22,7 @@ try:
 except ImportError:
     pass
 
-from monty.json import loadf as json_load
+from monty.json import loadf 
 from pymatgen.core.design_patterns import Enum, AttrDict
 from pymatgen.util.io_utils import FileLock
 from pymatgen.util.string_utils import stream_has_colours, is_string, list_strings, WildCard
@@ -122,7 +122,7 @@ class TaskResults(dict, MSONable):
 
     @classmethod
     def json_load(cls, filename):
-        return cls.from_dict(load(filename))
+        return cls.from_dict(loadf(filename))
 
 
 class ParalHintsError(Exception):
@@ -144,7 +144,6 @@ class ParalConf(AttrDict):
             version: 1
             autoparal: 1
             max_ncpus: 108
-
         configurations:
             -   tot_ncpus: 2         # Total number of CPUs
                 mpi_ncpus: 2         # Number of MPI processes.
@@ -202,16 +201,13 @@ class ParalHintsParser(object):
         """
         with abiinspect.YamlTokenizer(filename) as r:
             doc = r.next_doc_with_tag("!Autoparal")
-
-            #print(doc.text)
-            #with open(os.path.join(os.path.dirname(filename), "autoparal.yml"), "w") as fh:
-            #    fh.write(doc.text)
-
             try:
                 d = myaml.load(doc.text_notag)
                 return ParalHints(info=d["info"], confs=d["configurations"])
 
             except:
+                import traceback
+                print("traceback", traceback.format_exc())
                 raise self.Error("Wrong YAML doc:\n %s" % doc.text)
 
 
